@@ -4,7 +4,8 @@ In most cases it is advisable to use the higher-level quikkly.py wrapper instead
 """
 from __future__ import absolute_import
 
-from ctypes import cdll, Structure, Array, POINTER, byref, c_char_p, c_void_p, c_float, c_ubyte, c_uint8, c_int32, c_uint64, string_at, c_double
+from ctypes import cdll, Structure, Array, POINTER, byref, c_char_p, c_void_p, c_float, c_ubyte, c_uint8, c_int32, c_uint64, string_at, c_double, \
+    c_uint32
 import os
 import os.path
 import logging
@@ -13,11 +14,15 @@ import sys
 
 log = logging.getLogger(__name__)
 
+# Accepted native libs versions are MIN <= version < MAX
+LIB_VERSION_MIN = (3, 4, 5)  # Minimum native libs version for this SDK
+LIB_VERSION_MAX_ABOVE = (3, 5, 0)  # Future native libs version incompatible with this SDK - not inclusive.
+
 
-EXPECTED_LIB_VERSION = '3.4.5'
 QC_IMAGE_FORMAT_GREY_UINT8 = 0
 QC_IMAGE_FORMAT_BGRA_UINT32 = 1
 QC_IMAGE_FORMAT_RGBA_UINT32 = 2
+QC_IMAGE_FORMAT_NV21_UINT8 = 3  # Input byte buffer must be 1.5 * height rows!
 
 QC_IMAGE_FIT_DEFAULT = 0  # Template default
 QC_IMAGE_FIT_STRETCH = 1
@@ -91,8 +96,9 @@ class QCScanResult(PythonStructure):
     _pack_ = 1
     _fields_ = [
         ('tags', POINTER(QCTag)),
+        ('status_image', c_void_p),
         ('num_tags', c_int32),
-        ('_pad', c_int32),
+        ('status_code', c_int32),
     ]
 
     def to_python(self):
@@ -153,23 +159,28 @@ class Binary(object):
 
 BINARIES = [
     Binary('quikklycore', {
+        'qc_debug': (c_uint8, []),
         'qc_check_linking': (c_char_p, []),
 
         'qc_alloc_build_pipeline': (c_void_p, [c_char_p]),
         'qc_release_pipeline': (None, [c_void_p]),
 
+        'qc_num_templates': (c_uint32, [c_void_p]),
+        'qc_template_identifier': (c_char_p, [c_void_p, c_uint32]),
+
         'qc_process_frame': (c_int32, [c_void_p, POINTER(c_uint8), c_int32, c_int32, c_int32, c_int32]),
+        'qc_alloc_extract_result': (POINTER(QCScanResult), [c_void_p, c_uint8]),
+        'qc_release_result': (None, [POINTER(QCScanResult)]),
+
+        'qc_init_default_skin': (None, [POINTER(QCSkin)]),
+        'qc_template_exists': (c_uint8, [c_void_p, c_char_p]),
+        'qc_max_data_value': (c_uint64, [c_void_p, c_char_p]),
+        'qc_alloc_generate_svg': (c_void_p, [c_void_p, c_char_p, c_uint64, POINTER(QCSkin)]),
+        'qc_release_svg': (None, [c_void_p]),
+
         'qc_num_debug_images': (c_int32, [c_void_p, c_char_p]),
         'qc_access_debug_image': (POINTER(c_uint8), [c_void_p, c_char_p, c_int32, POINTER(c_int32), POINTER(c_int32), POINTER(c_int32)]),
         'qc_access_debug_seconds': (c_double, [c_void_p, c_char_p]),
-
-        'qc_alloc_extract_result': (POINTER(QCScanResult), [c_void_p]),
-        'qc_release_result': (None, [POINTER(QCScanResult)]),
-
-        'qc_max_data_value': (c_uint64, [c_void_p, c_char_p]),
-        'qc_init_default_skin': (None, [POINTER(QCSkin)]),
-        'qc_alloc_generate_svg': (c_void_p, [c_void_p, c_char_p, c_uint64, POINTER(QCSkin)]),
-        'qc_release_svg': (None, [c_void_p]),
     }),
 ]
 
@@ -190,10 +201,13 @@ def init(native_libs_dir=None, blueprint_file=None):
         FUNCS.update(binary.load(native_dir))
     if 'quikklycore.qc_check_linking' not in FUNCS:
         raise QuikklyError('Binary function quikklycore.qc_check_linking not loaded.')
-    loaded_version = FUNCS['quikklycore.qc_check_linking']()
-    if EXPECTED_LIB_VERSION.encode('utf-8') != loaded_version:
-        raise QuikklyError('Quikklycore binary version mismatch: loaded %s, but Python SDK is for %s.' % (loaded_version, EXPECTED_LIB_VERSION))
-    log.info('Quikkly native library version is %s', loaded_version)
+
+    loaded_version_str = FUNCS['quikklycore.qc_check_linking']().decode('utf-8')
+    loaded_version = tuple(map(int, loaded_version_str.split('.')))
+
+    if not (LIB_VERSION_MIN <= loaded_version < LIB_VERSION_MAX_ABOVE):
+        raise QuikklyError('Quikklycore binary version mismatch: loaded %s, but Python SDK is for %s ... < %s ' % (loaded_version_str, '.'.join(map(str, LIB_VERSION_MIN)), '.'.join(map(str, LIB_VERSION_MAX_ABOVE))))
+    log.info('Quikkly native library version is %s', loaded_version_str)
 
     if not blueprint_file:
         blueprint_file = os.path.join(native_dir, 'blueprint_default.json')
@@ -344,7 +358,7 @@ def _extract_results():
 
     Returns a Python dictionary with the results. Releases C library result before returning.
     """
-    results = FUNCS['quikklycore.qc_alloc_extract_result'](PIPELINE)
+    results = FUNCS['quikklycore.qc_alloc_extract_result'](PIPELINE, 0)
     results_python = results.contents.to_python()
     FUNCS['quikklycore.qc_release_result'](results)
     return results_python
